unit uSkinDraw;

{$MODE Delphi}

interface

uses
  SysUtils, opbitmapformats, lazbridge, Controls, Graphics, Classes, Forms, uSkinFile, IniFiles, ExtCtrls, uTypes, uFunk,

  Dialogs, uDebugger, LCLIntf;

type
  TSkinTrackBarScroll = procedure (i : Integer) of object;
  TSkinTrackBar = class
    constructor Create(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical, aMode : Boolean; aParent : TSkinTrackBarScroll; acSichtbar : Integer); // aMode: 0=TrackBar; 1=ScrollBar
    destructor Destroy; override;
  private
    Form                        : TForm;
    fTag                        : Integer;
    cSichtbar                   : Integer;
    Image                       : TImage;
    FlagMouse                   : Boolean;
    pwRegler, phRegler, hRegler, shRegler : Integer; // shRegler = Zuerst gesetzt Hhe ODER Breite des Reglers; hRegler = Hhe des Reglers, kann auch als wRegler eingesetzt werden
    Vertical, Mode              : Boolean;
    Scroll                      : TSkinTrackBarScroll;
    lItemWidth, hItemWidth      : Integer;
    procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
    Pos, uPos, Min, Max, Status, Width, Height : Integer; // uPos = umgerechnete Pos
    procedure Draw;
    procedure SetPosition(aPos : Integer);
    procedure SetMax(aMax : Integer);
  end;

  TSkin = class
    constructor Create(aForm : TForm; Skin : String);
    destructor Destroy; override;
  private
    ClientWidth, ClientHeight : Integer;
    FlagMouse                 : Boolean;
    fBTag                     : Integer;
    procedure LoadSkin(f : String);
  public
    function CreateTrackbar(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical : Boolean; aScroll : TSkinTrackBarScroll) : TSkinTrackBar;
    function CreateScrollbar(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical : Boolean; aScroll : TSkinTrackBarScroll; acSichtbar : Integer) : TSkinTrackBar;
  end;

implementation

uses
  code;

var tbOverComp : TSkinTrackBar;
    // Allgemeines
    Pic : Array of TBitmap;
    // Farben
    // Form
    cBG, cFont, cTrans : TColor; // Hintergrundfarbe, Textfarbe, Transparenz
    // Button
    cBNFont, cBOFont, cBDFont, cBIFont : TColor; // Textfarben - Normal, Drber, Gedrckt, Inaktiv (Disabled)
    // Listbox
    cLBActive, cLBActiveFont, cLBFocused, cLBFont : TColor; // Aktuelles Lied, Fokusiert, Textfarbe

procedure ClearComps;
begin
  tbOverComp := nil;
end;

// Skin

constructor TSkin.Create(aForm : TForm; Skin : String);
begin
  ClientWidth       := aForm.ClientWidth;
  ClientHeight      := aForm.ClientHeight;
  aForm.Color       := clWhite;

  if FileExists(Skin) then
  begin
    if Length(Pic) < 1 then //!
      LoadSkin(Skin);
  end else
    ; //!

  fBTag     := -1;
  FlagMouse := True;
end;

destructor TSkin.Destroy;
var i : Integer;
begin
  if Length(Pic) > 0 then
  begin
    for i := 0 to High(Pic) do
      Pic[i].Free; //! Muss Pic[i].Bitmap auch freigegeben werden ?
    SetLength(Pic, 0);
  end;

  if DirectoryExists(ExtractFilePath(Application.ExeName)+'\tmp') then
    RemoveDir(ExtractFilePath(Application.ExeName)+'\tmp');
end;

procedure TSkin.LoadSkin(f : String);
var Skin       : TSkinFile;
    sl         : TStrings;
    i, j       : Integer;
    fExt, fDat : String;
    fPic       : TOPPicture;
begin
  if not DirectoryExists(ExtractFilePath(Application.ExeName)+'\tmp') then
    CreateDir(ExtractFilePath(Application.ExeName)+'\tmp');

  sl := TStringList.Create;

  Skin := TSkinFile.Create(f);
  Skin.Files(sl);
  SetLength(Pic, 67);
  for i := 0 to High(Pic) do
    Pic[i] := TBitmap.Create;
  for i := 0 to Pred(sl.Count) do
  begin
    fExt := LowerCase(ExtractFileExt(sl[i]));
    for j := Length(sl[i]) downto 1 do
      if sl[i][j] = '.' then
        fDat := LowerCase(Copy(sl[i], 1, j-1));
    if (fExt = '.png') or (fExt = '.bmp') or (fExt = '.jpg') or (fExt = '.jpeg') then
    begin
      Skin.ExtractFile(ExtractFileName(sl[i]), ExtractFilePath(Application.ExeName)+'tmp\'+ExtractFileName(sl[i]));
      if not FileExists('tmp\'+sl[i]) then
      begin
        //!L
        Exit;
      end;
      fPic := TOPPicture.Create;
      fPic.LoadFromFile('tmp\'+sl[i]);

      if fDat = 'ctl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[0])
      else if fDat = 'ctr' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[1])
      else if fDat = 'bt' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[2])
      else if fDat = 'bl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[3])
      else if fDat = 'br' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[4])
      else if fDat = 'bb' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[5])
      else if fDat = 'bg' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[6])
      else if fDat = 'bu_n' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[7])
      else if fDat = 'bu_n_bl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[8])
      else if fDat = 'bu_n_br' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[9])
      else if fDat = 'bu_o' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[10])
      else if fDat = 'bu_o_bl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[11])
      else if fDat = 'bu_o_br' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[12])
      else if fDat = 'bu_d' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[13])
      else if fDat = 'bu_d_bl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[14])
      else if fDat = 'bu_d_br' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[15])
      else if fDat = 'tb_l' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[16]);
        //AssignOpBitmapToBitmap(fPic.Bitmap, Pic[17]);
        Pic[17] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_l_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[18]);
        //AssignOpBitmapToBitmap(fPic.Bitmap, Pic[19]);
        Pic[19] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_l_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[20]);
        //AssignOpBitmapToBitmap(fPic, Pic[21]);
        Pic[21] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_n' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[22]);
        //AssignOpBitmapToBitmap(fPic, Pic[23]);
        Pic[23] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_n_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[24]);
        //AssignOpBitmapToBitmap(fPic, Pic[25]);
        Pic[25] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_n_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[26]);
        //AssignOpBitmapToBitmap(fPic, Pic[27]);
        Pic[27] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_o' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[28]);
        //AssignOpBitmapToBitmap(fPic, Pic[29]);
        Pic[29] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_o_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[30]);
        //AssignOpBitmapToBitmap(fPic, Pic[31]);
        Pic[31] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_o_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[32]);
        //AssignOpBitmapToBitmap(fPic, Pic[33]);
        Pic[33] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_d' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[34]);
        //AssignOpBitmapToBitmap(fPic, Pic[35]);
        Pic[35] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_d_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[36]);
        //AssignOpBitmapToBitmap(fPic, Pic[37]);
        Pic[37] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'tb_r_d_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[38]);
        //AssignOpBitmapToBitmap(fPic, Pic[39]);
        Pic[39] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_l' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[40]);
        //AssignOpBitmapToBitmap(fPic, Pic[41]);
        Pic[41] := Rotate90(fPic.Bitmap);
      end else if fDat = 'scr_l_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[42]);
        //AssignOpBitmapToBitmap(fPic, Pic[43]);
        Pic[43] := Rotate90(fPic.Bitmap);
      end else if fDat = 'scr_l_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[44]);
        //AssignOpBitmapToBitmap(fPic, Pic[45]);
        Pic[45] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_n' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[46]);
        //AssignOpBitmapToBitmap(fPic, Pic[47]);
        Pic[47] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_n_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[48]);
        //AssignOpBitmapToBitmap(fPic, Pic[49]);
        Pic[49] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_n_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[50]);
        //AssignOpBitmapToBitmap(fPic, Pic[51]);
        Pic[51] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_o' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[52]);
        //AssignOpBitmapToBitmap(fPic, Pic[53]);
        Pic[53] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_o_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[54]);
        //AssignOpBitmapToBitmap(fPic, Pic[55]);
        Pic[55] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_o_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[56]);
        //AssignOpBitmapToBitmap(fPic, Pic[57]);
        Pic[57] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_d' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[58]);
        //AssignOpBitmapToBitmap(fPic, Pic[59]);
        Pic[59] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_d_bl' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[60]);
        //AssignOpBitmapToBitmap(fPic, Pic[61]);
        Pic[61] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'scr_r_d_br' then
      begin
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[62]);
        //AssignOpBitmapToBitmap(fPic, Pic[63]);
        Pic[63] := Rotate90(fPic.Bitmap);
      end else
      if fDat = 'lb' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[64])
      else if fDat = 'lb_bl' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[65])
      else if fDat = 'lb_br' then
        AssignOpBitmapToBitmap(fPic.Bitmap, Pic[66]);
    end else
    if sl[i] = 'index' then
    begin
      Skin.ExtractFile(ExtractFileName(sl[i]), ExtractFilePath(Application.ExeName)+'tmp\'+ExtractFileName(sl[i]));
      with TIniFile.Create(ExtractFilePath(Application.ExeName)+'tmp\index') do
      begin
        cBG           := StringToColor(ReadString('Skin', 'cFont'        , 'clBlack'  ));
        cFont         := StringToColor(ReadString('Skin', 'cFont'        , '$00C0C0C0'));
        cTrans        := StringToColor(ReadString('Skin', 'cTrans'       , '$00FF00FF'));
        cBNFont       := StringToColor(ReadString('Skin', 'cBNFont'      , 'clBlack'  ));
        cBOFont       := StringToColor(ReadString('Skin', 'cBOFont'      , 'clBlack'  ));
        cBDFont       := StringToColor(ReadString('Skin', 'cBDFont'      , 'clBlack'  ));
        cBIFont       := StringToColor(ReadString('Skin', 'cBIFont'      , '$00666666'));
        cLBActive     := StringToColor(ReadString('Skin', 'cLBActive'    , '$007D7D7D'));
        cLBActiveFont := StringToColor(ReadString('Skin', 'cLBActiveFont', '$00000000'));
        cLBFocused    := StringToColor(ReadString('Skin', 'cLBFocused'   , '$00FFFFFF'));
        cLBFont       := StringToColor(ReadString('Skin', 'cLBFont'      , '$007D7D7D'));

        Free;
      end;
    end;
  end;
  for i := 0 to High(Pic) do
  begin
    with Pic[i] do
    begin
      TransparentColor := cTrans;
      TransparentMode  := tmFixed;
    end;
  end;
  Skin.Free;
  sl.Free;
end;

// TrackBar

function TSkin.CreateTrackbar(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical : Boolean; aScroll : TSkinTrackBarScroll) : TSkinTrackBar;
begin
  Result := TSkinTrackBar.Create(aForm, aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler, aVertical, False, nil, -1);
end;

function TSkin.CreateScrollbar(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical : Boolean; aScroll : TSkinTrackBarScroll; acSichtbar : Integer) : TSkinTrackBar;
begin
  Result := TSkinTrackBar.Create(aForm, aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler, aVertical, True, aScroll, acSichtbar);
end;

constructor TSkinTrackBar.Create(aForm : TForm; aLeft, aTop, aWidth, aHeight, aMin, aMax, ahRegler : Integer; aVertical, aMode : Boolean; aParent : TSkinTrackBarScroll; acSichtbar : Integer); // aMode: 0=Trackbar; 1=Scrollbar
begin
  Form   := aForm;
  Scroll := aParent;
  Width  := aWidth;
  cSichtbar := acSichtbar;
  with aForm do
  begin
    Image := TImage.Create(aForm);
    with Image do
    begin
      Transparent := True;

      Parent := aForm;
      Left   := Pred(aLeft);
      Top    := Pred(aTop);
      Width  := Succ(aWidth);
      Height := Succ(aHeight);

      Status    := 0;
      FlagMouse := True;

      inc(fTag);
      Tag := fTag;

      OnMouseDown := MouseDown;
      OnMouseMove := MouseMove;
      OnMouseUp   := MouseUp;

      shRegler := ahRegler;

      Vertical := aVertical;
      Pos      := 0;
      Min      := aMin;
      if Vertical then //!
        Max := aMax
      else
        Max := aMax;
      dec(Max, cSichtbar);
      Mode := aMode;
    end;
    Draw;
  end;
end;

destructor TSkinTrackBar.Destroy;
begin
  //
end;

procedure TSkinTrackBar.Draw;
var tmpx, tmpxx, tmpy, tmpyy : Integer;
begin
  with Image.Canvas do
  begin
    Brush.Color := cTrans;
    Pen.Color   := cTrans;
    Rectangle(0, 0, Image.Width, Image.Height);

    Font.Color  := cFont;
    if Mode then
    begin
      if Vertical then
      begin
        tmpx  := (Image.Width div 2) div 2;
        tmpxx := Image.Width-tmpx;
        // Leiste
        // Flche
        StretchDraw(Rect(tmpx, Pic[43].Height, tmpxx, Pred(Image.Height-Pic[45].Height)), Pic[41]);
        // Rand oben
        StretchDraw(Rect(tmpx, 0, tmpxx, Pic[43].Height), Pic[43]);
        // Rand unten
        StretchDraw(Rect(tmpx, Pred(Image.Height-Pic[45].Height), tmpxx, Pred(Image.Height)), Pic[45]);

        tmpy := -1;
        // Regler
        if Status = 0 then
        begin
          // Flche
          tmpy := uPos+Pic[49].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+4), Pic[47]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[49]);
          // Rand unten
          tmpy := tmpy+4;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[51].Height), Pic[51]);

          hRegler := shRegler+Pic[49].Height+Pic[51].Height;
        end else
        if Status = 1 then
        begin
          // Flche
          tmpy := uPos+Pic[55].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+4), Pic[53]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[55]);
          // Rand unten
          tmpy := tmpy+4;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[57].Height), Pic[57]);

          hRegler := shRegler+Pic[55].Height+Pic[57].Height;
        end else
        if Status = 2 then
        begin
          // Flche
          tmpy := uPos+Pic[61].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+4), Pic[59]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[61]);
          // Rand unten
          tmpy := tmpy+4;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[63].Height), Pic[63]);

          hRegler := shRegler+Pic[61].Height+Pic[63].Height;
        end;
        phRegler := tmpy;
        if Height = -1 then
          Image.Height := Height-phRegler; //!
//          Height := Height-phRegler; //eFQ --> Height := Image.Height-phRegler

        Scroll(Pos);
      end else
      begin
        tmpx  := 0;
        tmpy  := (Image.Height div 2) div 2;
        tmpyy := Image.Height-tmpy;
        // Leiste
        // Flche
        StretchDraw(Rect(Succ(Pic[42].Width), tmpy, Pred(Image.Width-Pic[44].Width), tmpyy), Pic[40]);
        // Rand links
        StretchDraw(Rect(1, tmpy, Succ(Pic[42].Width), tmpyy), Pic[42]);
        // Rand rechts
        StretchDraw(Rect(Pred(Image.Width-Pic[44].Width), tmpy, Pred(Image.Width), tmpyy), Pic[44]);

        // Regler
        if Status = 0 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[48].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+4, Image.Height), Pic[46]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Height), Pic[48]);
          // Rand rechts
          tmpx := tmpx+4;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[50].Width, Image.Height), Pic[50]);

          hRegler := shRegler+Pic[48].Width+Pic[50].Width;
        end else
        if Status = 1 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[54].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+4, Image.Width), Pic[52]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Width), Pic[54]);
          // Rand rechts
          tmpx := tmpx+4;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[56].Width, Image.Width), Pic[56]);

          hRegler := shRegler+Pic[54].Width+Pic[56].Width;
        end else
        if Status = 2 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[60].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+4, Image.Width), Pic[58]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Width), Pic[60]);
          // Rand rechts
          tmpx := tmpx+4;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[62].Width, Image.Width), Pic[62]);

          hRegler := shRegler+Pic[60].Width+Pic[62].Width;
        end;
        pwRegler := tmpx;
        if Width = -1 then
          Image.Width := Width-pwRegler;

        Scroll(Pos);
      end;
    end else
    begin
      if Vertical then
      begin
        tmpx  := (Image.Width div 2) div 2;
        tmpxx := Image.Width-tmpx;
        // Leiste
        // Flche
        StretchDraw(Rect(tmpx, Pic[19].Height, tmpxx, Pred(Image.Height-Pic[21].Height)), Pic[17]);
        // Rand oben
        StretchDraw(Rect(tmpx, 0, tmpxx, Pic[19].Height), Pic[19]);
        // Rand unten
        StretchDraw(Rect(tmpx, Pred(Image.Height-Pic[21].Height), tmpxx, Pred(Image.Height)), Pic[21]);

        tmpy := -1;
        // Regler
        if Status = 0 then
        begin
          // Flche
          tmpy := uPos+Pic[25].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+8), Pic[23]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[25]);
          // Rand unten
          tmpy := tmpy+8;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[27].Height), Pic[27]);

          hRegler := shRegler+Pic[25].Height+Pic[27].Height;
        end else
        if Status = 1 then
        begin          // Flche
          tmpy := uPos+Pic[31].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+8), Pic[29]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[31]);
          // Rand unten
          tmpy := tmpy+8;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[33].Height), Pic[33]);

          hRegler := shRegler+Pic[31].Height+Pic[33].Height;
        end else
        if Status = 2 then
        begin
          // Flche
          tmpy := uPos+Pic[37].Height;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+8), Pic[35]);
          // Rand oben
          StretchDraw(Rect(0, uPos, Image.Width, tmpy), Pic[37]);
          // Rand unten
          tmpy := tmpy+8;
          StretchDraw(Rect(0, tmpy, Image.Width, tmpy+Pic[39].Height), Pic[39]);

          hRegler := shRegler+Pic[37].Height+Pic[39].Height;
        end;
        phRegler := tmpy;
        if Height = -1 then
          Image.Height := Image.Height-phRegler;
      end else
      begin
        tmpx  := 0;
        tmpy  := (Image.Height div 2) div 2;
        tmpyy := Image.Height-tmpy;
        // Leiste
        // Flche
        StretchDraw(Rect(Succ(Pic[18].Width), tmpy, Pred(Image.Width-Pic[20].Width), tmpyy), Pic[16]);
        // Rand links
        StretchDraw(Rect(1, tmpy, Succ(Pic[18].Width), tmpyy), Pic[18]);
        // Rand rechts
        StretchDraw(Rect(Pred(Image.Width-Pic[20].Width), tmpy, Pred(Image.Width), tmpyy), Pic[20]);

        // Regler
        if Status = 0 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[24].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+8, Image.Height), Pic[22]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Height), Pic[24]);
          // Rand rechts
          tmpx := tmpx+8;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[26].Width, Image.Height), Pic[26]);

          hRegler := shRegler+Pic[24].Height+Pic[26].Height;
        end else
        if Status = 1 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[30].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+8, Image.Height), Pic[28]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Height), Pic[30]);
          // Rand rechts
          tmpx := tmpx+8;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[32].Width, Image.Height), Pic[32]);

          hRegler := shRegler+Pic[30].Height+Pic[32].Height;
        end else
        if Status = 2 then
        begin
          // Flche
          tmpx := Succ(uPos+Pic[36].Width);
          StretchDraw(Rect(tmpx, 0, tmpx+8, Image.Height), Pic[34]);
          // Rand links
          StretchDraw(Rect(Succ(uPos), 0, tmpx, Image.Height), Pic[36]);
          // Rand rechts
          tmpx := tmpx+8;
          StretchDraw(Rect(tmpx, 0, tmpx+Pic[38].Width, Image.Height), Pic[38]);

          hRegler := shRegler+Pic[36].Height+Pic[38].Height;
        end;
        pwRegler := tmpx;
        if Width = -1 then
          Image.Width := Image.Width-pwRegler;
      end;
    end;
    with Image.Picture.Bitmap do
    begin
      TransparentColor := cTrans;
      TransparentMode  := tmFixed;
    end;
  end;
  tbOverComp := Self;
end;

procedure TSkinTrackBar.MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FlagMouse := False;
end;

procedure TSkinTrackBar.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var UserRange, SliderRange, aY : Integer; // aY kann auch als aX eingesetzt werden
    UserToIntern, InternToUser : Real;
begin
  ClearComps;
  tbOverComp := Self;

  if Vertical then
  begin
    if not FlagMouse then
    begin
      dec(Max, cSichtbar);

      UserRange:= Succ(Abs(Max-Min));
      if UserRange = 0 then
        UserRange := 1;

      SliderRange := Pred(Image.Height);
      if SliderRange = 0 then
        SliderRange := 1;

      UserToIntern := Pred(SliderRange)/Pred(UserRange);
      if UserToIntern = 0 then
        UserToIntern := 1;
      InternToUser := 1/UserToIntern;

      aY := Y+Image.Top;

      if aY < Image.Top then
        uPos := Image.Top
      else if aY > Pred(Pred(Image.Top+Image.Height)) then
        uPos := Pred(Pred(Image.Top+Image.Height))
      else
        uPos := aY;
      Pos  := Round((uPos-Image.Top)*InternToUser);

      uPos := uPos-Image.Top;
      if uPos < 0 then
        uPos := 0
      else if (Pos = Max) or (uPos >= Pred(Image.Height-hRegler)) then // ist die zweite If gefhrlich ?
        uPos := Pred(Image.Height-hRegler);
    end;
    if (Y >= uPos) and (Y <= phRegler) then
    begin
      if FlagMouse then
      begin
        Status := 1;
        Draw;
      end else
      begin
        Status := 2;
        Draw;
      end;
    end else
    begin
      Status := 0;
      Draw;
    end;
  end else
  begin
    if not FlagMouse then
    begin
      UserRange:= Succ(Abs(Max-Min));
      if UserRange = 0 then
        UserRange := 1;

      SliderRange := Pred(Image.Width);
      if SliderRange = 0 then
        SliderRange := 1;

      UserToIntern := Pred(SliderRange)/Pred(UserRange);
      if UserToIntern = 0 then
        UserToIntern := 1;
      InternToUser := 1/UserToIntern;

      aY := X+Image.Left;

      if aY < Image.Left then
        uPos := Image.Left
      else if aY > Pred(Pred(Image.Left+Image.Width)) then
        uPos := Pred(Pred(Image.Left+Image.Width))
      else
        uPos := aY;
      Pos  := Round((uPos-Image.Left)*InternToUser);

      aY := X+Image.Left;

      if aY < Image.Left then
        uPos := Image.Left
      else if aY > Pred(Pred(Image.Left+Image.Width)) then
        uPos := Pred(Pred(Image.Left+Image.Width))
      else
        uPos := aY;
      Pos  := Round((uPos-Image.Left)*InternToUser);
    end;
    if (X >= uPos) and (X <= pwRegler) then
    begin
      if FlagMouse then
      begin
        Status := 1;
        Draw;
      end else
      begin
        Status := 2;
        Draw;
      end;
    end else
    begin
      Status := 0;
      Draw;
    end;
  end;
end;

procedure TSkinTrackBar.MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FlagMouse := True;
  Status    := 0;
  Draw;
end;

procedure TSkinTrackBar.SetPosition(aPos : Integer);
begin
  Pos := aPos;
  Draw;
end;

procedure TSkinTrackBar.SetMax(aMax : Integer);
begin
  Max := aMax-cSichtbar;
end;

end.
